@@ -1,11 +1,10 @@
-/* Copyright (C) 1998 Thorsten Kukuk
+/* Copyright (C) 1998, 1999, 2001 Thorsten Kukuk
    This file is part of the yp-tools.
-   Author: Thorsten Kukuk <kukuk@vt.uni-paderborn.de>
+   Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   it under the terms of the GNU General Public License version 2 as 
+   published by the Free Software Foundation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,6 +20,11 @@
 #include "config.h"
 #endif
 
+#define keydat_val dptr
+#define keydat_len dsize
+#define valdat_val dptr
+#define valdat_len dsize
+
 #include <pwd.h>
 #ifdef HAVE_CRYPT_H
 #include <crypt.h>
@@ -31,16 +35,27 @@
 #else
 #include "lib/getopt.h"
 #endif
+#include <netdb.h>
+#include <limits.h>
 #include <string.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <locale.h>
 #include <libintl.h>
 #include <sys/param.h>
 #include <rpcsvc/yp_prot.h>
 #include <rpcsvc/ypclnt.h>
+#ifdef HAVE_RPC_CLNT_SOC_H
+#include <rpc/clnt_soc.h>
+#endif
 
 #include "yppasswd.h"
 
+#if defined (__NetBSD__) || (defined(__GLIBC__) && (__GLIBC__ == 2 && __GLIBC_MINOR__ == 0))
+/* <rpc/rpc.h> is missing the prototype */
+int getrpcport(char *host, int prognum, int versnum, int proto);
+#endif
+
 #ifndef _
 #define _(String) gettext (String)
 #endif
@@ -222,7 +237,7 @@ ypgetpw (char *master, char *domainname, char *name, int uid)
   struct ypreq_key key;
   enum clnt_stat res;
   struct ypresp_val resp;
-  char buffer[1024], uidbuf[8], *ptr, *keyval;
+  char *buffer, uidbuf[256], *ptr, *keyval;
   static struct passwd pwd;
   char *map;
   CLIENT *clnt;
@@ -232,7 +247,9 @@ ypgetpw (char *master, char *domainname, char *name, int uid)
 
   if (name == NULL)
     {
-      sprintf (uidbuf, "%d", uid);
+      if (snprintf (uidbuf, sizeof (uidbuf), "%d", uid) > sizeof (uidbuf))
+	abort ();
+
       keyval = strdup (uidbuf);
       map = strdup ("passwd.byuid");
     }
@@ -241,6 +258,7 @@ ypgetpw (char *master, char *domainname, char *name, int uid)
       keyval = strdup (name);
       map = strdup ("passwd.byname");
     }
+
   /* Fill in values. */
   memset (&resp, 0, sizeof (struct ypresp_val));
   key.domain = strdup (domainname);
@@ -267,6 +285,7 @@ ypgetpw (char *master, char *domainname, char *name, int uid)
       return NULL;
     }
 
+  buffer = alloca (resp.valdat.valdat_len + 1);
   strncpy (buffer, resp.valdat.valdat_val, resp.valdat.valdat_len);
   buffer[resp.valdat.valdat_len] = '\0';
   free (resp.valdat.valdat_val);
@@ -361,9 +380,9 @@ verifypassword (struct passwd *pwd, char *pwdstr, uid_t uid)
   other = ucase = lcase = 0;
   for (p = pwdstr; *p; p++)
     {
-      ucase = ucase || isupper (*p);
-      lcase = lcase || islower (*p);
-      other = other || !isalpha (*p);
+      ucase = ucase || isupper ((int)*p);
+      lcase = lcase || islower ((int)*p);
+      other = other || !isalpha ((int)*p);
     }
 
   if ((!ucase || !lcase) && !other && uid)
@@ -427,6 +446,13 @@ main (int argc, char **argv)
   else
     progname = argv[0];
 
+  if ((strcmp (progname, "ypchsh") == 0) ||
+      (strcmp (progname, "chsh") == 0))
+    l_flag = 1;
+  else if ((strcmp (progname, "ypchfn") == 0) ||
+	   (strcmp (progname, "chfn") == 0))
+    f_flag = 1;
+
   while (1)
     {
       int c;
@@ -440,7 +466,7 @@ main (int argc, char **argv)
       };
 
       c = getopt_long (argc, argv,
-		       (strcmp ("yppasswd", progname) == 0) ? "flp?" : "?",
+		       (l_flag == 0 && f_flag == 0) ? "flp?" : "?",
 		       long_options, &option_index);
       if (c == (-1))
         break;
@@ -456,9 +482,9 @@ main (int argc, char **argv)
 	  p_flag = 1;
 	  break;
 	case '?':
-	  if (strcmp (progname, "ypchsh") == 0)
+	  if (l_flag)
 	    print_help_chsh ();
-	  else if (strcmp (progname, "ypchfn") == 0)
+	  else if (f_flag)
 	    print_help_chfn ();
 	  else
 	    print_help_pwd ();
@@ -467,17 +493,17 @@ main (int argc, char **argv)
 	  print_version (progname);
 	  return 0;
 	case '\254':
-	  if (strcmp (progname, "ypchsh") == 0)
+	  if (l_flag)
 	    print_usage_chsh (stdout);
-	  else if (strcmp (progname, "ypchfn") == 0)
+	  else if (f_flag)
 	    print_usage_chfn (stdout);
 	  else
 	    print_usage_pwd (stdout);
 	  return 0;
 	default:
-	  if (strcmp (progname, "ypchsh") == 0)
+	  if (l_flag)
 	    print_usage_chsh (stderr);
-	  else if (strcmp (progname, "ypchfn") == 0)
+	  else if (f_flag)
 	    print_usage_chfn (stderr);
 	  else
 	    print_usage_pwd (stderr);
@@ -488,11 +514,6 @@ main (int argc, char **argv)
   argc -= optind;
   argv += optind;
 
-  if (strcmp (progname, "ypchsh") == 0)
-    l_flag = 1;
-  else if (strcmp (progname, "ypchfn") == 0)
-    f_flag = 1;
-
   if (f_flag == 0 && l_flag == 0)
     p_flag = 1;
 
@@ -519,7 +540,8 @@ main (int argc, char **argv)
   uid = getuid ();
   if ((pwd = ypgetpw (master, domainname, user, uid)) == NULL)
     {
-      fprintf (stderr, _("%s: unknown user (uid=%d).\n"), progname, uid);
+      fprintf (stderr, _("%s: unknown user (uid=%ld).\n"), progname,
+	       (long)uid);
       return 1;
     }
 
@@ -552,7 +574,7 @@ main (int argc, char **argv)
   if ((pwd->pw_passwd != NULL && strlen (pwd->pw_passwd) > 0) ||
       (pwd->pw_uid != uid))
     {
-      char prompt[40];
+      char prompt[130];
 
       if (pwd->pw_uid != uid)
         sprintf (prompt, _("Please enter root password:"));
@@ -582,7 +604,7 @@ main (int argc, char **argv)
       int tries = 0;
       time_t tm;
 
-      buf = (char *) malloc (30);
+      buf = (char *) malloc (129);
 
       printf (_("Changing NIS password for %s on %s.\n"), pwd->pw_name,
 	      master);
@@ -604,7 +626,8 @@ main (int argc, char **argv)
 	      return 1;
 	    }
 
-	  strcpy (buf, p);
+	  strncpy (buf, p, 128);
+	  buf[128] = '\0';
 
 #ifdef USE_CRACKLIB
 	  error_msg = FascistCheck (buf, CRACKLIB_DICTPATH);
@@ -639,6 +662,7 @@ main (int argc, char **argv)
     {
       char gecos[1024], *sp, new_gecos[1024];
       char name[254], location[254], office[254], phone[254];
+      char oname[254], olocation[254], ooffice[254], ophone[254];
 
       printf (_("\nChanging full name for %s on %s.\n"
 	     "To accept the default, simply press return. To enter an empty\n"
@@ -646,20 +670,20 @@ main (int argc, char **argv)
 	      pwd->pw_name, master);
 
       strncpy (gecos, pwd->pw_gecos, sizeof (gecos));
-      sp = getfield (gecos, name, sizeof (name));
-      if (newfield (progname, _("Name"), strtok (gecos, ","), name,
+      sp = getfield (gecos, oname, sizeof (oname));
+      if (newfield (progname, _("Name"), oname, name,
 		    sizeof (name)))
 	return 1;
-      sp = getfield (sp, location, sizeof (location));
-      if (newfield (progname, _("Location"), location, location,
+      sp = getfield (sp, olocation, sizeof (olocation));
+      if (newfield (progname, _("Location"), olocation, location,
 		    sizeof (location)))
 	return 1;
-      sp = getfield (sp, office, sizeof (office));
-      if (newfield (progname, _("Office Phone"), office, office,
+      sp = getfield (sp, ooffice, sizeof (ooffice));
+      if (newfield (progname, _("Office Phone"), ooffice, office,
 		    sizeof (office)))
 	return 1;
-      sp = getfield (sp, phone, sizeof (phone));
-      if (newfield (progname, _("Home Phone"), phone, phone, sizeof (phone)))
+      sp = getfield (sp, ophone, sizeof (ophone));
+      if (newfield (progname, _("Home Phone"), ophone, phone, sizeof (phone)))
 	return 1;
       sprintf (new_gecos, "%s,%s,%s,%s", name, location, office, phone);
       sp = new_gecos + strlen (new_gecos);
@@ -720,9 +744,10 @@ main (int argc, char **argv)
 		 _("\nThe GECOS information has not been changed on %s.\n\n"),
 		   master);
 	}
-      fprintf (stdout,
-	       _("\nThe GECOS information has been changed on %s.\n\n"),
-	       master);
+      else
+	fprintf (stdout,
+		 _("\nThe GECOS information has been changed on %s.\n\n"),
+		 master);
       break;
     case 4:
       if (error || status)
@@ -735,8 +760,9 @@ main (int argc, char **argv)
 		   _("\nThe login shell has not been changed on %s.\n\n"),
 		   master);
 	}
-      fprintf (stdout,
-	       _("\nThe login shell has been changed on %s.\n\n"), master);
+      else
+	fprintf (stdout,
+		 _("\nThe login shell has been changed on %s.\n\n"), master);
       break;
     default:
       if (error || status)
@@ -750,9 +776,10 @@ main (int argc, char **argv)
 	   _("\nThe NIS account information has not been changed on %s.\n\n"),
 		   master);
 	}
-      fprintf (stdout,
-	       _("\nThe NIS account information has been changed on %s.\n\n"),
-	       master);
+      else
+	fprintf (stdout,
+		 _("\nThe NIS account information has been changed on %s.\n\n"),
+		 master);
       break;
     }
 
